; ------------------------------------------------------------------------
; Define ZP variables 

.zeropage

zpstart	= *
tmp1 = 0 ;	      	.res	1	; $00  
tmp2 = 1 ;:	      	.res	1	; $01  
tmp3 = 2 ;:	      	.res	1	; $02  |
tmp4 = 3 ;:	      	.res	1	; $03  |
ptr1 = 4 ;:	      	.res	2	; $04  |
ptr2 = 6 ;:	      	.res	2	; $06  |
ptr3 = 8 ;:	      	.res	2	; $08  |
endzp:


; ------------------------------------------------------------------------
; ------------------------------------------------------------------------

.segment	"INIT"

; ------------------------------------------------------------------------
; ------------------------------------------------------------------------

.code

; ------------------------------------------------------------------------
; ------------------------------------------------------------------------
; 				main program
; ------------------------------------------------------------------------
; ------------------------------------------------------------------------

_start:
		ldx	#$FF	; Set a valid stack pointer for simulation
		txs
		lda	$C050	; graphics
		lda	$C053	; Mixed
		lda	$C054	; Page 1
		lda	$C057	; Hires
		sta $C00F	; Altchar
		
		;jmp tests
		
		lda	$C0E8	; disk motor OFF

		; fill RAM (mainly to avoid a warm boot)
		ldy #0
		sty ptr1
		sty ptr1+1
		ldy	#8
		lda	#$20	; space character
l0:		sta (ptr1),y
		iny
		bne	l0
		inc	ptr1+1
		lda	#$20	; up to graphic video areas ($2000)
		cmp	ptr1+1
		bne	l0
		lda	#$FF
		sta	tmp2

		ldx #0		; puts-like print
lp1:	lda	txtpool,x
		beq	l01
		;and #$3F	; Inverse video, no flash
		sta	1876,x
		inx
		bne	lp1		; always taken

txtpool:
txt1:	.asciiz "IIe BOOT <any key> or upload SNAP"

		; random pattern for the rest of memory (tmp2:3 starts with $2020)
l01:	asl tmp2	; 16-bit, $1021 polynomial
		rol tmp3
		bcc	l02
		lda	#$21	
		eor tmp2
		sta	tmp2
		lda	#$10
		eor tmp3
		sta	tmp3
l02:	lda tmp3
		sta (ptr1),y
		iny
		bne	l01
		inc	ptr1+1
		lda	#$C0	;end of RAM?
		cmp	ptr1+1
		bne	l01
		
		; Load ROMs from SPI Flash (address: $7FC000)
		; $7E0000: Defender		(26k)
		; $7E8000: Apple-IIe	(16k)
		
		; $7F0000: ZX spectrum	(16K)
		; $7F4000: Amstrad CPC	(32K)
		; $7FC000: Apple-II		(16K - 256 bytes)
		
		lda	#3		; Read command to Flash
		jsr	spibyte	; spibyte also sets /CS low
		lda	#$7E
		jsr	spibyte
		lda	#$80
		jsr	spibyte
		lda	#$00
		jsr	spibyte

		sta $C006	; Enable slot roms
		ldy #0
l21:	jsr	spibyte	; Read data
		sta $C600,y	; write slot #6 ROM
		iny
		bne	l21

		sta $C007	; disable slot ROMs
		lda	#$C1
		sta	ptr1+1
		ldy #0
		sty ptr1

l2:		jsr	spibyte	; Read data
		sta (ptr1),y
		iny
		bne	l2
		inc	ptr1+1
		bne	l2
		sta $C006	; Enable slot roms
		lda	#$1C	; set /CS high
		sta	$C020
		
		; snap or boot
		jsr getch
		sta	tmp3
		cmp	#$A5	; 6502 snap
		beq snap
		cmp #$A8	; Z80 snap
		bne boot1
		; Apple II boot
		; copy trampoline to RAM (to address $0000)

		; Loap snapshot (4KB max) to $1000
snap:	jsr getch	
		sta	tmp1
		jsr getch
		sta tmp2	; tmp[2:1] = nbytes
		lda #$10
		sta ptr1+1
		ldy #0
		sty ptr1
l3:		jsr getch
		sta (ptr1),y
		iny
		bne l4
		inc ptr1+1
l4:		clc
		lda tmp1
		sbc #0
		sta tmp1
		lda tmp2
		sbc #0
		sta tmp2
		ora tmp1
		bne l3
		
		; Copy boot code to RAM
boot1:	ldx	#(tpl2end-tpl2-1)
cpy2:	lda	tpl2,x
		sta	$10,x
		dex
		bpl	cpy2
		jmp	$10		;and jump to it
		
		; Code to run from RAM
		; must be position independent
tpl2:	lda #$0c	; no BOOT, no FAST
		sta $C020
		lda	tmp3
		cmp #$A5
		bne	l5
		jmp $1000	; exec 6502 code
l5:		cmp #$A8
		beq	l6
		jmp	($FFFC)	; jump to Reset vector
l6:		sta $C400	; start Z80
		ldx #0		; puts-like print
l7:		lda	txtnoz80-tpl2+$10,x
		beq	l6
		sta	1876,x
		inx
		bne	l7		; always taken

txtnoz80: ;.asciiz "             No Z80              "
          .asciiz "         No Z80  (halt)          "
tpl2end:
		;--------------------------------
		; SPI bitbanging
		;--------------------------------
spibyte:
		ldx	#8
		sta	tmp1
spb1:	asl	tmp1
		lda	#$38	; SCK=0, /CS=0,
		adc	#0		; MOSI=carry
		sta	$C020	
		ora #2		; SCK=1
		sta	$C020	
		bit	$C020	; N flag = MISO
		bpl	spb2
		inc	tmp1
spb2:	dex
		bne	spb1
		lda	#$38	; SCK=0, /CS=0 
		sta	$C020
		lda	tmp1
return:	rts

; getch
getch:
lgch:	bit	$C000	; wait for valid data
		bpl	lgch
		lda	$C010	; read it and clear flag
		rts

;----------------------------------------------
tests:
		lda #$1C	; no fast
		sta $C020
		lda	$C0E9	; disk motor ON
		lda	$C0EF	; write
		nop
		ldx	#3
			
ndat:	lda #$FF	;2	
		sta	$C0ED	;4
		cmp $C0EC	;4

		pha			;3
		pla			;4

		jsr	return	;12
		
		nop
		nop
		nop
		dex
		bne	ndat
		
		ldx	#2
mdat:	lda #$FC	;2
		sta	$C0ED	;4
		cmp $C0EC	;4

		pha			;3
		pla			;4

		jsr	return	;12
		dex
		bne	mdat
		
		lda	$C0EE	; read

cuel9:	jmp cuel9

_nmi:
_irqbrk:

	.export endtxt
endtxt:

; ------------------------------------------------------------------------
; ------------------------------------------------------------------------
; 				Vectors
; ------------------------------------------------------------------------
; ------------------------------------------------------------------------
	.segment	"VECTORS"
	.export	vectorhNMI, vectorhRES, vectorhIRQBRK
vectorhNMI:	
	.byte <_nmi	;NMI
	.byte >_nmi
vectorhRES:
	.byte <_start	;RESET 
	.byte >_start
vectorhIRQBRK:
	.byte <_irqbrk	;IRQ/BRK
	.byte >_irqbrk
