#include <stdlib.h>
#include <stdio.h>

#define BPT (6656)
#define BTRK (6392)
#define DSZ (BPT*35)
unsigned char dsk[DSZ];

unsigned char lut[256];

const unsigned char encodetab[64] ={
0x96, 0x97, 0x9a, 0x9b, 0x9d, 0x9e, 0x9f, 0xa6,
0xa7, 0xab, 0xac, 0xad, 0xae, 0xaf, 0xb2, 0xb3,
0xb4, 0xb5, 0xb6, 0xb7, 0xb9, 0xba, 0xbb, 0xbc,
0xbd, 0xbe, 0xbf, 0xcb, 0xcd, 0xce, 0xcf, 0xd3,
0xd6, 0xd7, 0xd9, 0xda, 0xdb, 0xdc, 0xdd, 0xde,
0xdf, 0xe5, 0xe6, 0xe7, 0xe9, 0xea, 0xeb, 0xec,
0xed, 0xee, 0xef, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6,
0xf7, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff	
};


void hdump(unsigned char *buf,int max)
{
	int i,j,k;
	
	for (i=0;i<256; i+=16) {
		printf("%03X  ",i);
		k=((max-i)>16)? 16 : max-i;
		for (j=0;j<k;j++) printf("%02X ",buf[i+j]);
		for (;j<16;j++) printf("   ");
		for (j=0;j<k;j++) putchar((buf[i+j]>=' ' && buf[i+j]<128)?buf[i+j]:'.');
		putchar('\n');
		if (k<16) break;
	}
}

unsigned char *ptrk;
int bitix=0;
unsigned char d1,d2,d3;
int readbyte()
{
	int i,d,bi;
	d=0;
	do {
		d<<=1;
		bi=bitix; while (bi>=BTRK*8) bi-=BTRK*8;
		if (ptrk[bi>>3]&(1<<(7-(bi&7)))) d++;
		bitix++;
		if (bitix==BTRK*8) printf(" -- ");
		
	} while ((d&0x80)==0 && bitix<12*BTRK);
	d3=d2; d2=d1; d1=d;
	return d;
}


main(int argc, char **argv)
{
	int i,j,k,v,s,t,c,x,a;
	unsigned char buf[256],buf2[86];
	unsigned char buftrk[16][256];
	//const unsigned char skew[16]=  { 0,13,11,9,7,5,3,1,14,12,10,8,6,4,2,15 };
	const unsigned char skewr[16]= { 0,7,14,6,13,5,12,4,11,3,10,2,9,1,8,15 };
	FILE *fp;
	
	if (argc<2) {
		printf(
			"Usage: %s <input.img>\n"
			" %s reads a raw Apple II floppy disk image like the real system\n"
			" looking for the address fields of sectors (D5 AA 96 marks),\n"
			" decoding the information, and checking the checksums.\n"
			" The retrieved data is stored in the file \"dec.dsk\"\n"
			,argv[0],argv[0]);
		exit(1);
	}

	if ((fp=fopen(argv[1],"rb"))==NULL) exit(1);
	fread(dsk,1,sizeof(dsk),fp);
	fclose(fp);
	
	if ((fp=fopen("dec.dsk","wb"))==NULL) exit(1);
	
	for (i=0;i<256;i++) lut[i]=0x80;
	for (i=0;i<64;i++) lut[encodetab[i]]=i;
	
	for (i=0;i<35;i++) {
		printf("T%3d: ",i);
		ptrk=&dsk[i*BPT];
		bitix=0;
		for(j=0;j<16;j++) {
			// First The address
			do {
				readbyte();
				if (bitix>=BTRK*12) goto nexttrack;

			}while (d3!=0xD5 || d2!=0xAA || d1!=0x96);
			
			v=(readbyte()<<1)+1;
			v&=readbyte();
			t=(readbyte()<<1)+1;
			t&=readbyte();
			s=(readbyte()<<1)+1;
			s&=readbyte();
			c=(readbyte()<<1)+1;
			c&=readbyte();

			if (t!=i) printf("(T%d) ",t);
			printf("%2d",s);
			//printf("$%02X %2d %2d $%02X %s ",v,t,s,c, (v^t^s^c)?"Error":"Ok");
			
			// Now the data
			do {
				readbyte();
				if (bitix>=BTRK*12) goto nexttrack;
			}while (d3!=0xD5 || d2!=0xAA || d1!=0xAD);
			
			a=x=0;
			for (k=85;k>=0;k--) {	// 2bits
				x=lut[readbyte()];
				a^=x;
				buf2[k]=a;
			}
			
			for (k=0;k<256;k++) {	// 6bits
				a^=lut[readbyte()];
				buf[k]=a;
			}
			//putchar('\n');
			//hdump(buf,256); putchar('\n'); hdump(buf2,86);
			//exit(0);
			
			a^=lut[readbyte()];	// checksum
			k='.';
			if (a) k='E'; else if(v^t^s^c) k='e';
			printf("%c ",k);
			if (!a) {
				for(k=0,c=86;k<256;k++) {
					if (--c<0) c=85;
					buf[k]<<=2;
					if (buf2[c]&1) buf[k]+=2;
					if (buf2[c]&2) buf[k]+=1;
					buf2[c]>>=2;
				}

				memcpy(&buftrk[skewr[s]][0],buf,256);
				if (t==0 && s<16) {
					//hdump(buf); putchar('\n');
				}
			}
			
		}
nexttrack:
		fwrite(buftrk,1,16*256,fp);
		putchar('\n');
	}
	fclose(fp);
	printf("Decoded disk written to: \"dec.dsk\"\n");
}
